// ****************************************************************************
// Copyright (C) 1998 by Dieter Windau
// All rights reserved
//
// BmpMenu.cpp: implementation file
// Version:     1.2
// Date:        07.08.1998
// Author:      Dieter Windau
//
// TBmpMenu is a freeware OWL class that shows bitmaps
// on the left side of menus like MS Office 97
//
// Portions of code are based on MFC class CMenuSpawn written by Iuri
// Apollonio and BCMenu written by Brent Corkum. Very special thanks.
//
// You are free to use/modify this code but leave this header intact.
// May not be sold for profit.
//
// Tested with Borland C++ 5.02, OWL 5.02 with Windows NT 4.0 SP3 but I think
// the class would work with Windows 95 too.
// This file is provided "as is" with no expressed or implied warranty.
// Use at your own risk.
//
// Please send me bug reports, bug fixes, enhancements, requests, etc., and
// I'll try to keep this in next versions:
//   EMail: dieter.windau@usa.net
//   Web:   http://www.members.aol.com/SoftEngage
// ****************************************************************************

#ifndef __BMPMENU_H
#define __BMPMENU_H

#if defined(BI_PLAT_WIN32)

#ifndef   OWL_IMAGELST_H
#include <owl\imagelst.h>
#endif
#ifndef   CLASSLIB_ARRAYS_H
#include <classlib\arrays.h>
#endif

const MaxMenuStringLen    = 128;

// *********************** helper functions ***********************************

// MFC style function, that draw the frame of a 3d rect with given colors
void Draw3dRect(HDC hdc, const TRect& rect,
	COLORREF clrTopLeft, COLORREF clrBottomRight);
void Draw3dRect(HDC hdc, int x, int y, int cx, int cy,
	COLORREF clrTopLeft, COLORREF clrBottomRight);

// MFC style function, that draws a filled rect. Is this faster???
void FillSolidRect(HDC hdc, int x, int y, int cx, int cy, COLORREF clr);

// ********************* TBmpMenuItem *****************************************

class TBmpMenuItem
{
public:
  HMENU Menu;
  int		ImageIdx;
  int		Cmd;
  UINT  Pos;
  TCHAR	Text[MaxMenuStringLen];

	bool operator ==(const TBmpMenuItem& d) const
    { return memcmp(this, &d, sizeof(d)) == 0 ? true : false; }
};

// ********************* TBmpMenuImageItem ************************************

class TBmpMenuImageItem
{
public:
  int		ImageIdx;
  int		Cmd;

	bool operator ==(const TBmpMenuImageItem& d) const
    { return memcmp(this, &d, sizeof(d)) == 0 ? true : false; }
};

// *************************** TBmpMenu ***************************************

class TBmpMenu
{
public:
	TBmpMenu(TModule* module = 0);
	~TBmpMenu();

public:
	void RemapMenu(HMENU hMenu, bool isPopup=false);
  // Remap a simple menu created with Windows API functions or with OWL TMenu
  // class to a menu with bitmaps on the left side. Should be called from e.g:
  // SetupWindow(), EvInitMenu(), EvInitMenuPopup() functions and before the
  // TrackPopupMenu() function.
  // If the hMenu is a popupMenu set isPopup = true

  bool DrawItem(DRAWITEMSTRUCT far& DIS);
  // To be called from the TWindow EvDrawItem(..)
  // Call the base class if it return false

	bool MeasureItem(MEASUREITEMSTRUCT far& MIS);
  // To be called from the TWindow EvMeasureItem(..)
  // Call the base class if it return false

	bool FindKeyboardShortcut(UINT nChar, UINT nFlags, HMENU hMenu, LRESULT& lRes);
  // To be called from the TWindow EvMenuChar(..)
  // Call the base class if it return false

  int PreGetHintText(HMENU hmenu, uint id, LPTSTR buf, int size, THintText hintType);
  // Retrieves the hint text associated with a particular Id
  // To be called from DecoratedFrame GetHintText(...)
  // Call the base class if it return 0.

  bool IsDrawRecessed() { return bDrawRecessed; }
  void SetDrawRecessed(bool b) { bDrawRecessed = b; }
  // Get/Set the style how to draw top level items

  bool IsWithAccel() { return bShowAccInTooltips; }
  void SetWithAccel(bool b) { bShowAccInTooltips = b; }
  // Get/Set the showing of accelerators in tooltips

  bool IsBackBitmap() { return bBackBitmap; }
	bool SetBackBitmap(unsigned int resId);
  // Set the background bitmap. If resId == 0 the menus shown normal.
  // return true if successful

  void ExcludeBitmap(unsigned int resId);
  // Exclude the bitmap with resId. It will not be shown in menus
  void IncludeBitmap(unsigned int resId);
  // Include the bitmap with resId. By default all bitmaps in module
  // will be shown in menus

  // ------ Some simple functions to change menu on the fly
  //
  // If you want to change more, use OWL or Windows API function.
  // But then you must remap the menu.
  //
  bool SetMenuItemText(HMENU hMenu, UINT pos, const char* txt);
  // Set the menu item text like Win API function:
  //   ModifyMenu(hMenu, pos, MF_BYPOSITION | MF_STRING, .., txt)

protected:
	void RemapMenuRecursiv(HMENU hMenu);
	void DrawDisabledButton(TDC& dc, const TRect& rc);
  void DrawCheckmark(TDC& hdc, int x, int y, COLORREF color);
  void DrawRadioDot(TDC& dc, int x, int y, COLORREF color);
  int  FindImageItem(const int cmd);
	TBmpMenuItem* AddMenuItem(HMENU hMenu, LPCTSTR txt, int cmd, UINT pos);
	void AddImageItem(int idx, WORD cmd);
	bool TryToLoadBmpResource(unsigned int resId);
  void SyncronizeAllMenuItems();

protected:
  bool        bShowAccInTooltips; // show accelerators in tooltips, e.g. New (Strg+N)
  bool        bDrawRecessed;    // show top level menu items recessed
  bool        bIsPopup;         // true if the remap menu is a popup menu
  bool        bBackBitmap;      // show a background bitmap
	TImageList* ImageList;        // contains all used bitmaps (each bmp one time)
	TSize 			ImageSize;        // the size that is used for all bitmaps
  TBitmap*    BackBitmap;       // background bitmap
  COLORREF    Old3dFaceColor;
  TModule*    Module;

#if (OWLInternalVersion >= 0x06000000L)
  TArrayAsVector<unsigned int>        ExcludedBitmaps;
	TIArrayAsVector<TBmpMenuItem*>      MenuItems;
  TIArrayAsVector<TBmpMenuImageItem*> ImageItems;
#else
  TIArrayAsVector<unsigned int>       ExcludedBitmaps;
	TIArrayAsVector<TBmpMenuItem>       MenuItems;
  TIArrayAsVector<TBmpMenuImageItem>  ImageItems;
#endif
};

#endif

#endif
